@@ -28,6 +28,8 @@
  */
 
 #include <sys/types.h>
+#include <sys/stat.h>
+#include <fcntl.h>
 #include <unistd.h>
 #include <stdlib.h>
 
@@ -88,9 +90,21 @@ mmc_monitor_dir(int idx)
 static void 
 hw_state_event_cb(osso_hw_state_t *hw_state, gpointer data)
 {
+    int fd;
 	if (hw_state->shutdown_ind == TRUE) {
 		sending_enabled = 0;
 		osso_log(LOG_DEBUG, "[%s]:Shutting down", __FUNCTION__);
+		/*
+		 * create flag in /tmp for rich-core-dumper,
+		 * because we have about 2-sec window (on Rover)
+		 * from here to actual runlevel-based shutdown during when 
+		 * X server refuses to serve and apps may abort due to gdk_x_error
+		 * and we don't want to have such coredumps
+		 */
+		fd = creat("/tmp/do_not_dump_rich_cores", S_IRUSR|S_IRGRP|S_IROTH);
+		if (fd != -1) {
+		    close(fd);
+		}
 		/*
 		 * There is possibility that daemon sends RPC about new rcore
 		 * to UI, which goes processing (and remains in rpc call)
@@ -212,8 +226,8 @@ ssc=`dbus-send --system --print-reply --dest=com.nokia.phone.SSC /com/nokia/phon
 sig-strength=`dbus-send --system --print-reply --dest=com.nokia.phone.net /com/nokia/phone/net Phone.Net.get_signal_strength | tail -n +2 | awk '{print $2}'`,\
 gprs-status=`dbus-send --system --print-reply --dest=com.nokia.csd.GPRS /com/nokia/csd/gprs com.nokia.csd.GPRS.GetStatus | tail -n +2 | awk '$1~/string|boolean|uint64/ {print $2}'`,\
 gprs-serv-status=`dbus-send --print-reply --system --dest=com.nokia.csd.GPRS /com/nokia/csd/gprs org.freedesktop.DBus.Properties.GetAll string: | tail -n +2 | awk '$2~/boolean|uint64/{print $3}'`,\
-gprs-rx-B=`gconftool-2 --get /system/osso/connectivity/network_type/GPRS/gprs_rx_bytes`,\
-gprs-tx-B=`gconftool-2 --get /system/osso/connectivity/network_type/GPRS/gprs_tx_bytes`,\
+gprs-rx-B=,\
+gprs-tx-B=,\
 call-counters=`echo 'select * from Events where Events.service_id = 1;' | sqlite3 /home/user/.rtcom-eventlogger/el.db | \
     awk -F'|' '{c[$3]++ }END{printf(\"inbound=%d outbound=%d missed=%d\",c[1],c[2],c[3])}'`,"
 
@@ -362,12 +376,10 @@ lifelog_add_event(char *event, char *datastr)
     char cmd[512];
     int rv;
 
-    sprintf(cmd, "echo LL_%s date=`date +%%s`, %s >> %s", event, datastr, LIFELOG_FILE);
-    rv = system(cmd);
-    /*
-    osso_log(LOG_DEBUG, "[%s]: rv=%d from cmd[%s]", 
-	     __FUNCTION__, rv, cmd);
-    */
+    if (lifelog_enabled) {
+	sprintf(cmd, "echo LL_%s date=`date +%%s`, %s >> %s", event, datastr, LIFELOG_FILE);
+	rv = system(cmd);
+    }
 }
 
 #ifndef RUNS_IN_SDK_HOST
