@@ -153,7 +153,6 @@ send_rich_core(gchar * filename)
 	}
 	
  	if(!core_is_valid(filename)) {
-		osso_log(LOG_INFO,"%s is ignored", filename);
 		return;
 	}
 		
@@ -163,6 +162,8 @@ send_rich_core(gchar * filename)
 	    unlink(filename);
 	    return;
 	}
+	lifelog_add_event("COREDUMP", filename);
+
 	if (!sending_enabled) {
 	    return;
 	}
@@ -174,14 +175,11 @@ send_rich_core(gchar * filename)
 	    return;
 	}
 	strncpy(prev_corename, filename, MAX_PREV_CORENAME_SZ - 1);
-	lifelog_add_event("COREDUMP", filename);
 
 	osso_log(LOG_INFO, "invoke ui for %s", filename);
 	retval = osso_rpc_run_with_defaults(context, CREPORTER_SERVICE, 
 					    CREPORTER_METHOD_SEND_FILE, NULL, 
 					    DBUS_TYPE_STRING, filename, DBUS_TYPE_INVALID);
-
-	return;
 }
 
 /**
