@@ -63,7 +63,7 @@ static GKeyFile *creporter_load_settings_key_file()
 	    return NULL;
 	}
 
-	osso_log(LOG_DEBUG, "server settings loaded from '%s'", filename);
+	osso_log(LOG_DEBUG, "settings from '%s'", filename);
 	return key;
 }
 
@@ -115,7 +115,7 @@ static GKeyFile *creporter_load_privacy_settings_key_file()
 	}
     }
 
-    osso_log(LOG_DEBUG, "privacy settings loaded from '%s'", filename);
+    osso_log(LOG_DEBUG, "settings from '%s'", filename);
     return key;
 }
 
