@@ -130,8 +130,11 @@ core_is_valid (gchar *filename)
 	/*Ignore the partially created cores,
 	 * these have the prefix '.'*/
 	rv = FALSE;
-    } else if(g_strrstr(filename, CRASH_REPORTER_UI)) {
-	/* Ignore crash_reporter_ui cores*/
+    } else if(g_strrstr_len(filename, 14, CRASH_REPORTER_UI)) {
+	/* Ignore crash_reporter_ui cores, but only match by first 14
+	 * chars, because core dumper creates file name using first 15 chars,
+	 * the file name is like this: crash_reporter_-5E5E-6-3492.rcore.lzo 
+	 */
 	rv = FALSE;
     } else if(g_str_has_suffix(basename, ".rcore") ||
 	      g_str_has_suffix(basename, ".rcore.lzo")) {
