@@ -104,7 +104,7 @@ static osso_return_t
 creporter_ui_rpc_callback_handler(const gchar * interface, const gchar * method, 
 			      GArray * arguments, gpointer method_name, osso_rpc_t *retval)
 {
-    gchar *filename = NULL;
+    gchar *new_elem = NULL;
     GSList *temp = NULL;
 	
     if ((method == NULL) || (interface == NULL)) {
@@ -126,16 +126,18 @@ creporter_ui_rpc_callback_handler(const gchar * interface, const gchar * method,
 	 * this happens when user note gets added, then lzo file
 	 * gets overwritten with same name
 	 */
+
 	for(temp = files_list; temp ; temp = g_slist_next(temp)) {
-	    if( !strcmp((gchar *)temp->data, g_array_index(arguments, osso_rpc_t, 0).value.s)) {
+	    if( !strcmp(CORE_FILE_NAME(temp), g_array_index(arguments, osso_rpc_t, 0).value.s)) {
 		osso_log(LOG_DEBUG, "[%s]: skip add %s (exists)", 
-			 __FUNCTION__, (gchar *)temp->data);
+			 __FUNCTION__, CORE_FILE_NAME(temp));
 		return OSSO_OK;
 	    }
 	}
 	/* Appending single file to the list*/
-	filename = g_strdup(g_array_index(arguments, osso_rpc_t, 0).value.s);
-	files_list = g_slist_prepend(files_list, filename);
+	new_elem = g_malloc0(sizeof(struct send_sel_data) + strlen(g_array_index(arguments, osso_rpc_t, 0).value.s) + 1);
+	strcpy(new_elem + sizeof(struct send_sel_data), g_array_index(arguments, osso_rpc_t, 0).value.s);
+	files_list = g_slist_prepend(files_list, new_elem);
 	if(creporter_ui_status == NOT_ALREADY_UP) {
 	    creporter_ui_status = SEND_FILE;
 	    creporter_ui_set_connec_callback_notifiers();
@@ -187,7 +189,6 @@ main(int argc, char *argv[])
 	
 	/* Initialization functions */
 	g_thread_init(NULL);
-	osso_log(LOG_DEBUG, "Crash Reporter UI is launched with %d arg\n", argc);
 	context = osso_initialize(CRASH_REPORTER_UI, "1.0", TRUE, NULL);
 	g_return_val_if_fail(context != NULL, 0);
 
