@@ -27,7 +27,10 @@
  */
 
 #include <gconf/gconf-client.h>
+#include <hildon/hildon.h>
+#include <hildon/hildon-caption.h>
 #include <hildon/hildon-note.h>
+#include <hildon/hildon-pannable-area.h>
 #include <unistd.h>
 #include <glib.h>
 #include <stdlib.h> 
@@ -59,7 +62,6 @@ extern GSList *files_list;
 /* Crash Reporter UI Application context */
 extern osso_context_t *context;
 
-
 /**
   This function set the connection status callback notifiers 
   
@@ -77,12 +79,15 @@ void creporter_ui_set_connec_callback_notifiers();
 */
 static gboolean creporter_ui_collect_all_corefile_names_at_location(const gchar *location);
 
-void usecase_send_button_clicked (GtkWidget *button, GtkTextBuffer *buffer);
-void usecase_other_button_clicked (GtkWidget *button, void *arg);
-void usecase_options_button_clicked (GtkWidget *button, void *arg);
+static void usecase_send_button_clicked (GtkWidget *button, GtkTextBuffer *buffer);
+static void usecase_reuse_button_clicked (GtkWidget *button, GtkTextBuffer *buffer);
+static void usecase_save_note_button_clicked (GtkWidget *button, GtkTextBuffer *buffer);
+static void usecase_other_button_clicked (GtkWidget *button, void *arg);
+static void usecase_options_button_clicked (GtkWidget *button, void *arg);
 static GtkWidget *creporter_ui_usecase_dialog(gchar *text);
 static void creporter_ui_process_response(gboolean send_all_files_mode, gint response);
 static gchar *compose_crashed_note(gchar *filename);
+static GtkWidget *creporter_ui_select_for_sending_dialog(gint num);
 
 /*
  * counts and stats
@@ -104,17 +109,52 @@ static GtkWidget* progressbar = NULL;
 static GtkWidget* progresslabel = NULL;
 static double progress_percent = 0.0;
 
+GtkWidget *reuse_button = NULL;
+
 #define PROGRESS_DLG_VISIBLE	(progressbar && progresslabel)
-#define LIST_MAX_CORES (8)
+
+/*
+ * declrs and defs for selected upload/delete stuff
+ */
+#define MAX_SENDING_SHOW (300)
+#define MAX_CORE_NAMELEN_FITS_TO_BUTTON (40)
+#define PANNING_THRESHOLD 30 
+#define MARGIN_HALF 4 
+#define MARGIN_DEFAULT 8
+
+gint num_of_select_send_buttons = 0;
+creporterSettings *gsettings = NULL;
+static gchar * create_server_label_text(gint num_of_checked, gint sum_size, gchar * addr);
+static void select_all_button_clicked (GtkWidget *button, void *arg);
+GtkWidget *glabel_server;
+GtkWidget *selection_checkall_button;
+GtkWidget *selection_uncheckall_button;
+GtkWidget *selection_send_button;
+GtkWidget *selection_delete_button;
+gint sel_num_of_checked = 0;
+gint sel_sum_size = 0;
+
+/*
+ * declrs and defs for stored re-usable use case notes.
+ */
+#define USECASE_NOTES_STORAGE	"/home/user/.crash_reporter_usecasenotes"
+#define USECASE_NOTES_MAX_SAVED	(50)
+
+static void usecase_notes_ringbuf_fromfile(void);
+static void usecase_notes_ringbuf_tofile(void);
+static void usecase_notes_add_one(gchar *);
+static GSList *usecase_notes_list = NULL;
+static GSList *usecase_notes_curr = NULL;
 
 static void 
 creporter_ui_process_response(gboolean send_all_files_mode, gint response)
 {
-    gchar *file_path=NULL;
+    struct send_sel_data *sel;
+    GSList *temp;
 
     switch(response) {
     case GTK_RESPONSE_CANCEL:
-	osso_log(LOG_DEBUG, "[%s]: User cancelled\n", __FUNCTION__);
+	osso_log(LOG_DEBUG, "User cancelled");
 	creporter_ui_exit();
 	break;
 	
@@ -122,7 +162,7 @@ creporter_ui_process_response(gboolean send_all_files_mode, gint response)
 	/* Create an internet connection and wait for gconf connec callback*/
 	if (!creporter_connect_iap()) {
 	    creporter_ui_display_note(CONF_CONNECTION_ERROR, NULL);
-	    osso_log(LOG_DEBUG, "[%s]: could not connect\n", __FUNCTION__);
+	    osso_log(LOG_DEBUG, "[%s]: could not connect", __FUNCTION__);
 	    creporter_ui_exit();
 	}
 	break;
@@ -131,24 +171,43 @@ creporter_ui_process_response(gboolean send_all_files_mode, gint response)
      */	
     case GTK_RESPONSE_APPLY:
 	if(files_list) {
-	    osso_log(LOG_DEBUG, "[%s]: User selected Delete\n", __FUNCTION__);
+	    osso_log(LOG_DEBUG, "User selected Delete");
 	    if(send_all_files_mode) {
-		GSList *temp = files_list;
-		while(temp){
-		    if(g_remove(temp->data) != 0)
+		for (temp = files_list; temp ; temp = g_slist_next(temp)){
+		    sel = CORE_SELECT_CTX(temp);
+		    if (num_of_select_send_buttons) {
+			if(sel->button && !sel->sel_state) {
+			    /* button but not selected for action */
+			    continue;
+			}
+			if(!sel->button) {
+			    /* no button for this, but there are buttons,
+			       means there was too many entries, but this was not selected */
+			    continue;
+			}
+		    }
+		    if(g_remove(CORE_FILE_NAME(temp)) != 0) {
 			osso_log(LOG_DEBUG,"[%s] Could not remove the file [%s] errno=%d", 
-				 __FUNCTION__, temp->data, errno);
-		    temp = g_slist_next(temp);
+				 __FUNCTION__, CORE_FILE_NAME(temp), errno);
+		    }
+		    g_free(temp->data);
 		}
 		g_slist_free(files_list);
 		files_list = NULL;
 	    } else {
-		file_path = (gchar *)files_list->data;
-		if(g_remove(file_path) != 0)
+		/*
+		 * should check here for selection data for completeness,
+		 * but in current code single core deletion never can have
+		 * selection of cores, so here this one is not checked
+		 */ 
+		void *p;
+		if(g_remove(CORE_FILE_NAME(files_list)) != 0)
 		    osso_log(LOG_DEBUG,"[%s] Could not remove the core file [%s], errno=%d",
-			     __FUNCTION__, file_path, errno);
-		g_free(file_path);
+			     __FUNCTION__, CORE_FILE_NAME(files_list), errno);
+		p = files_list->data;
 		files_list = g_slist_remove(files_list, files_list->data);
+		g_free(p);
+		p = NULL;
 	    }
 	}
 	creporter_ui_exit();
@@ -218,7 +277,6 @@ creporter_ui_send_progress_dialog ()
     g_timeout_add(1000, update_progressbar, NULL);
 
     /*------------*/
-
     /* Add a "Stop" button to the bottom of the dialog */
     cancel_button = gtk_dialog_add_button(GTK_DIALOG(window),
 					  dgettext("hildon-libs",
@@ -246,7 +304,6 @@ static gint
 creporter_ui_upload_files(crash_upload_context ** pctx)
 {
     GSList *temp = NULL;
-    temp = files_list;
     gint rv = -1;
     gint response;
     GtkWidget *wid = NULL;
@@ -257,37 +314,68 @@ creporter_ui_upload_files(crash_upload_context ** pctx)
     GThread* send_thread = NULL;
     creporterSettings *settings = NULL;
     crash_upload_context * ctx = NULL;
+    struct send_sel_data *sel;
 
     settings = creporter_get_settings();
     if (!settings) {
-	osso_log(LOG_DEBUG, "[%s]: Unable to read settings!\n", __FUNCTION__);
+	osso_log(LOG_DEBUG, "[%s]: Unable to read settings", __FUNCTION__);
 	return CREPORTER_NO_SETTINGS_ERROR;
     } else {
-	osso_log(LOG_DEBUG, "[%s]: Settings: '%s'-'%s'-'%s'\n", 
+	osso_log(LOG_DEBUG, "[%s]: Settings: '%s'-'%s'-'%s'", 
 		 __FUNCTION__, settings->server_addr,
 		 settings->user_name, settings->passwd);
     }
 
     ctx = g_new0(crash_upload_context, 1);
     ctx->settings = settings;
-    tot = g_slist_length(files_list);
+    /*
+     * if selection was made, total number is by selected set,
+     * otherwise it is length of list
+     */
+    if (num_of_select_send_buttons) {
+	tot = sel_num_of_checked;
+    } else {
+	tot = g_slist_length(files_list);
+    }
+
     tried_count = succeeded_count = 0;
     if (!send_progress_dialog) {
 	wid = creporter_ui_send_progress_dialog();
 	send_progress_dialog = GTK_DIALOG(wid);
 	gtk_window_set_title(GTK_WINDOW(send_progress_dialog), _("Sending crash report"));
     }
-    for(cur = 1, temp = files_list; temp; temp = g_slist_next(temp), cur++) {
+    /*
+     * cur:counts cores we send (for display)
+     */
+    for(cur = 0, temp = files_list; temp; temp = g_slist_next(temp)) {
+
 	/*
 	 * Skip processing if there is no file (means, it was just sent
 	 * and deleted on previous loop round)
 	 */
-	if(access(temp->data, R_OK) < 0) {
-	    osso_log(LOG_DEBUG, "[%s]: File %s not found, skip", __FUNCTION__, temp->data);
+	if(access(CORE_FILE_NAME(temp), R_OK) < 0) {
+	    osso_log(LOG_DEBUG, "[%s]: File %s not found, skip", __FUNCTION__, CORE_FILE_NAME(temp));
 	    continue;
 	}
 
-	strncpy(ctx->filename, temp->data, sizeof(ctx->filename) - 1);
+	/*
+	 * selected upload support: if we have selections state,
+	 * check it and skip if entry was not selected
+	 */
+	sel = CORE_SELECT_CTX(temp);
+	if (num_of_select_send_buttons) {
+	    if(sel->button && !sel->sel_state) {
+		/* button but not selected for action */
+		continue;
+	    }
+	    if(!sel->button) {
+		/* no button for this, but there are buttons,
+		   means there was too many entries, but this was not selected */
+		continue;
+	    }
+	}
+	strncpy(ctx->filename, CORE_FILE_NAME(temp), sizeof(ctx->filename) - 1);
+	cur += 1;
 
 	if (send_thread) {
 	    osso_log(LOG_DEBUG, "[%s] previous upload thread still active, wait...", __FUNCTION__);
@@ -299,11 +387,11 @@ creporter_ui_upload_files(crash_upload_context ** pctx)
 
 	tried_count += 1;
 	if (PROGRESS_DLG_VISIBLE) {
-	    if(stat((gchar *)temp->data, &file_info) < 0) {
-		osso_log(LOG_DEBUG, "[%s]: can  not stat %s, skip", __FUNCTION__, temp->data);
+	    if(stat(CORE_FILE_NAME(temp), &file_info) < 0) {
+		osso_log(LOG_DEBUG, "[%s]: can  not stat %s, skip", __FUNCTION__, CORE_FILE_NAME(temp));
 		continue;
 	    }
-	    basename = g_filename_display_basename((gchar *)temp->data);
+	    basename = g_filename_display_basename(CORE_FILE_NAME(temp));
 	    if (tot > 1) {
 		gchar * sending_note = g_strdup_printf(_("Sending crash report %d / %d"), cur, tot);
 		gtk_window_set_title (GTK_WINDOW (send_progress_dialog), sending_note);
@@ -327,7 +415,6 @@ creporter_ui_upload_files(crash_upload_context ** pctx)
 	}
 	if (response == GTK_RESPONSE_CANCEL) {
 	    /*  cancel or dialog delete */
-	    osso_log(LOG_DEBUG, "[%s] cancel/delete (%d), delete dialog", __FUNCTION__, response);
 	    gtk_widget_destroy(GTK_WIDGET(wid));
 	    send_progress_dialog = NULL;
 	    progressbar = NULL;
@@ -481,7 +568,7 @@ compose_crashed_note (gchar *filename)
     if (sig_num && exe_name && process_id) {
 	text = g_strdup_printf(_("'%s' PID %s died to signal %s"), exe_name, process_id, sig_num);
     } else {
-	text = g_strdup_printf(_("A crash file %s was generated"), (gchar *) files_list->data);
+	text = g_strdup_printf(_("A crash file %s was generated"), filename);
     }
     text2 = g_strdup_printf(_("%s. Send %d kB crash report for analysis?"), 
 			    text, size/1024); 
@@ -494,43 +581,6 @@ compose_crashed_note (gchar *filename)
     return text2;
 }
 
-static gint 
-list_first_cores(gchar ** listed)
-{
-    gint cnt;
-    gint size = 0;
-    GSList *temp = NULL;
-    gchar * s = NULL;
-    gchar * s2 = NULL;
-    struct stat file_info;
-    gchar *basename;
-
-    s = g_strdup("");
-    for(cnt = 0, temp = files_list; temp ; temp = g_slist_next(temp), cnt++) {
-	bzero(&file_info, sizeof(file_info));
-	if(stat((gchar *)temp->data, &file_info) < 0) {
-	    continue;
-	}
-	size += file_info.st_size;
-	if (cnt <= LIST_MAX_CORES - 1) {
-	    basename = g_filename_display_basename((gchar *)temp->data);
-	    s2 = g_strdup_printf(_("%s%ld kB\t%s\n"), s, file_info.st_size/1024, basename);
-	    g_free(s);
-	    g_free(basename);
-	    s = s2;
-	}
-    }
-    if (cnt > LIST_MAX_CORES) {
-	s2 = g_strdup_printf(_("%s--showing first %d of %d entries--\n"), 
-			     s, LIST_MAX_CORES, cnt);
-	g_free(s);
-	s = s2;
-    }
-
-    *listed = s;
-    return size;
-}
-
 /**
   This function is used to display all the confirmation notes in the appl.
 
@@ -545,10 +595,8 @@ creporter_ui_display_note (creporterConfirmationType conf_type, crash_upload_con
     gint response = 0;
     creporterSettings *sett = NULL;
     gint num;
-    gchar * listed = NULL;
-    gint size;
     gchar *basename = NULL;
-	
+
     if (dialog) {
 	osso_log(LOG_DEBUG, "[%s]: dialog already active, skip and return", 
 		 __FUNCTION__);
@@ -557,6 +605,7 @@ creporter_ui_display_note (creporterConfirmationType conf_type, crash_upload_con
 	}
 	return response;
     }
+    num_of_select_send_buttons = 0;
     switch(conf_type) {
     case CONF_USER_CONFIRMATION1:
 	if(!files_list) {
@@ -566,31 +615,15 @@ creporter_ui_display_note (creporterConfirmationType conf_type, crash_upload_con
 	    }
 	    return GTK_RESPONSE_CANCEL;
 	}
-	text = compose_crashed_note((gchar *) files_list->data);
+	usecase_notes_ringbuf_fromfile();
+	text = compose_crashed_note(CORE_FILE_NAME(files_list));
 	note = creporter_ui_usecase_dialog(text);
 	break;
     case CONF_USER_CONFIRMATION2:
 	sett = creporter_get_settings();
 	num = g_slist_length(files_list);
 	if (num > 0) {
-	    size = list_first_cores(&listed);
-	    text = g_strdup_printf(_("\
-This system has %u stored crash %s:\n\
-%s\
-Total %d kB, send for analysis?\n\
-%s%s"),
-				   num, (num>1)? "reports" : "report",
-				   listed, size/1024,
-				   sett? "\nReceiving server: " : "", 
-				   sett? sett->server_addr : "");
-
-	    g_free(listed);
-	    listed = NULL;
-	    note = hildon_note_new_confirmation_add_buttons(NULL, text, 
-							    _("Yes"), GTK_RESPONSE_OK,
-							    _("No"),GTK_RESPONSE_CANCEL, 
-							    _("Delete All"), GTK_RESPONSE_APPLY,
-							    NULL);
+	    note = creporter_ui_select_for_sending_dialog(num);
 	} else {
 	    text = g_strdup(_("This system has no stored crash reports."));
 	    note = hildon_note_new_information(NULL, text);
@@ -659,7 +692,14 @@ file %s, problem connecting to server, send status %d"),
         
     if (note) {
 	dialog = GTK_DIALOG(note);
-	response = gtk_dialog_run(dialog);
+	/*
+	 * This loop makes repeated dialog via RESPONSE_YES return value, 
+	 * used for example from "send all" dialog when Select/Deselect ALL us used
+	 */
+	do {
+	    response = gtk_dialog_run(dialog);
+	} while (response == GTK_RESPONSE_YES);
+
 	dialog = NULL;
 	gtk_widget_destroy(GTK_WIDGET(note));
 	note = NULL;
@@ -670,11 +710,146 @@ file %s, problem connecting to server, send status %d"),
     if (ctx) {
 	g_free(ctx);
     }
+    reuse_button = NULL;
     return response;
 }
 
-/* Callback for close button */
-void
+/* 
+ * Store re-usable use case notes to a file.
+ * NULL-terminated strings are stored in file one after other,
+ * NULL-char included.
+ */
+static void
+usecase_notes_ringbuf_tofile ()
+{
+    gchar *buf = NULL;
+    gsize len, thislen;
+    GSList *temp;
+    gint cnt;
+
+    if (usecase_notes_list) {
+	len = cnt = 0;
+	for (temp = usecase_notes_list; temp ; temp = g_slist_next(temp)){
+	    thislen = strlen((gchar*)temp->data);
+	    buf = g_realloc(buf, len + thislen + 1);
+	    strcpy(buf + len, (gchar*)temp->data);
+	    len += thislen + 1;
+	    /*
+	     * we limit endless growth of list by saving max N entries.
+	     * Because we only add at head, this discards older entries.
+	     */
+	    if( ++cnt >= USECASE_NOTES_MAX_SAVED ) {
+		break;
+	    }
+	}
+	g_file_set_contents(USECASE_NOTES_STORAGE, buf,
+			    len, NULL);
+    }
+}
+
+/* 
+ * Read re-usable use case notes from a file.
+ * NULL-terminated strings are in file one after other,
+ * NULL-char included.
+ */
+static void
+usecase_notes_ringbuf_fromfile ()
+{
+    gchar *buf = NULL;
+    gsize len;
+    gchar *p, *end;
+    gboolean rv = g_file_get_contents(USECASE_NOTES_STORAGE, &buf,
+				      &len, NULL);
+    if (rv) {
+	p = buf;
+	end = p + len;
+	while (p < end) {
+	    usecase_notes_list = g_slist_prepend(usecase_notes_list, g_strdup(p));
+	    usecase_notes_curr = usecase_notes_list;
+	    while (p < end && *p) {
+		p++;
+	    }
+	    p++;
+	}
+	g_free(buf);
+	if (reuse_button && usecase_notes_list && usecase_notes_curr) {
+	    gtk_widget_set_sensitive(reuse_button, TRUE);
+	}
+    }
+}
+
+/*
+ * insert text to list of re-usable usecase notes
+ * Note. this will store argument "text" 
+ * directly in list without making a copy, so
+ * caller of this should not free it after call.
+ */
+static void 
+usecase_notes_add_one(gchar *text)
+{
+    GSList *temp;
+
+    if (strlen(text) == 0) {
+	return;
+    }
+    for (temp = usecase_notes_list; temp ; temp = g_slist_next(temp)){
+	/*
+	 * only if such entry not on list already.
+	 */
+	if( !g_strcmp0(text, (gchar*)temp->data)) {
+	    g_free(text);
+	    return;
+	}
+    }
+    usecase_notes_list = g_slist_prepend(usecase_notes_list, text);
+    usecase_notes_curr = usecase_notes_list;
+    usecase_notes_ringbuf_tofile();
+    if (usecase_notes_list && usecase_notes_curr) {
+	gtk_widget_set_sensitive(reuse_button, TRUE);
+    }
+}
+
+/*
+ * re-use one of previously used/stored use case comments
+ */
+static void
+usecase_reuse_button_clicked (GtkWidget *button, GtkTextBuffer *buffer)
+{
+    if (usecase_notes_list && usecase_notes_curr) {
+	gtk_text_buffer_set_text(buffer, (gchar *)usecase_notes_curr->data, -1);
+	if(usecase_notes_curr->next) {
+	    usecase_notes_curr = usecase_notes_curr->next;
+	} else {
+	    usecase_notes_curr = usecase_notes_list;
+	}
+    }
+}
+
+/* 
+ * "add current note into ring-buffer of saved notes" button 
+ */
+static void
+usecase_save_note_button_clicked (GtkWidget *button, GtkTextBuffer *buffer)
+{
+    GtkTextIter start;
+    GtkTextIter end;
+    gchar *text;
+    
+    /* Obtain iters for the start and end of points of the buffer */
+    gtk_text_buffer_get_start_iter (buffer, &start);
+    gtk_text_buffer_get_end_iter (buffer, &end);
+
+    /* Get the entire buffer text. */
+    text = gtk_text_buffer_get_text (buffer, &start, &end, FALSE);
+
+    /*
+     * this eats text and keeps it so we are not leaking it here
+     */
+    usecase_notes_add_one(text);
+}
+
+/* Callback for Send button */
+static void
 usecase_send_button_clicked (GtkWidget *button, GtkTextBuffer *buffer)
 {
     GtkTextIter start;
@@ -701,20 +876,23 @@ usecase_send_button_clicked (GtkWidget *button, GtkTextBuffer *buffer)
 			 __FUNCTION__, RICH_CORE_TMP_NOTEFILE);
 	    }
 	    fclose(fp);
-	    cmd_len = sizeof(LZOP_PROG) + sizeof(RICH_CORE_TMP_NOTEFILE) + strlen((gchar *)files_list->data) + 20;
+	    cmd_len = sizeof(LZOP_PROG) + sizeof(RICH_CORE_TMP_NOTEFILE) + strlen(CORE_FILE_NAME(files_list)) + 20;
 	    cmd = g_malloc(cmd_len);
 	    if (cmd) {
-		sprintf(cmd, "%s -c %s >> %s", LZOP_PROG, RICH_CORE_TMP_NOTEFILE, (gchar *)files_list->data);
+		sprintf(cmd, "%s -c %s >> %s", LZOP_PROG, RICH_CORE_TMP_NOTEFILE, CORE_FILE_NAME(files_list));
 		res = system(cmd);
-		osso_log(LOG_ERR,"[%s]: system[%s] returned %d\n", __FUNCTION__, cmd, res);
 		g_free(cmd);
 	    }
 	    unlink(RICH_CORE_TMP_NOTEFILE);
 	} else {
-	    osso_log(LOG_ERR,"[%s]: error opening file %s\n", __FUNCTION__, RICH_CORE_TMP_NOTEFILE);
+	    osso_log(LOG_ERR,"[%s]: error opening file %s", __FUNCTION__, RICH_CORE_TMP_NOTEFILE);
 	}
+	/*
+	 * add it also to list of saved notes for re-use.
+	 * Note. this will keep text used in list so no need to free it
+	 */
+	usecase_notes_add_one(text);
     }
-    g_free (text);
 
     /*
      * If we are here, then dialog must exist and
@@ -727,7 +905,7 @@ usecase_send_button_clicked (GtkWidget *button, GtkTextBuffer *buffer)
 }
 
 /* Callback for options button */
-void
+static void
 usecase_options_button_clicked (GtkWidget *button, void *arg)
 {
     gint res = 0;
@@ -736,12 +914,12 @@ usecase_options_button_clicked (GtkWidget *button, void *arg)
     if (context && top_window) {
 	res = osso_cp_plugin_execute(context, "libcpcrashreporter.so", top_window, TRUE);
     } else {
-	osso_log(LOG_DEBUG, "[%s]: no osso context or topwindow??",__FUNCTION__);
+	osso_log(LOG_DEBUG, "[%s]: no osso context or topwindow?",__FUNCTION__);
     }
 }
 
 /* Callback for other buttons */
-void
+static void
 usecase_other_button_clicked (GtkWidget *button, void *arg)
 {
     gint response = (gint)arg;
@@ -766,11 +944,12 @@ creporter_ui_usecase_dialog (gchar *text)
     GtkWidget *scrolled_window;
     GtkWidget *label_note;
     GtkWidget *label_server;
+    GtkWidget *save_note_button;
     GtkWidget *send_button;
     GtkWidget *delete_button;
     GtkWidget *options_button;
-    GtkTextBuffer *buffer;
     GtkWidget *text_view;
+    GtkTextBuffer *buffer;
     GtkWidget *frame;
     creporterSettings *sett = NULL;
     
@@ -804,7 +983,6 @@ creporter_ui_usecase_dialog (gchar *text)
                         TRUE, TRUE, 0);
     gtk_container_add (GTK_CONTAINER (frame), scrolled_window);
 
-
     /* Create a multiline text widget. */
     text_view = gtk_text_view_new ();
     gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(text_view), GTK_WRAP_WORD);
@@ -826,6 +1004,25 @@ creporter_ui_usecase_dialog (gchar *text)
 			    FALSE, FALSE, 0);
     }
     /*------------*/
+    /* "Re-use note" button */
+    reuse_button = gtk_dialog_add_button(GTK_DIALOG(window), _("Previous note"),
+					GTK_RESPONSE_YES);
+    g_signal_connect (G_OBJECT (reuse_button), "clicked",
+		      G_CALLBACK (usecase_reuse_button_clicked),
+		      buffer);
+    if (!(usecase_notes_list && usecase_notes_curr)) {
+	gtk_widget_set_sensitive(reuse_button, FALSE);
+    }
+    
+    /*------------*/
+    /* "Save note as template" button */
+    save_note_button = gtk_dialog_add_button(GTK_DIALOG(window), _("Keep this note"),
+					     GTK_RESPONSE_YES);
+    g_signal_connect (G_OBJECT (save_note_button), "clicked",
+		      G_CALLBACK (usecase_save_note_button_clicked),
+		      buffer);
+    
+    /*------------*/
 
     /* FIXME: Now that the buttons are created with gtk_dialog_add_button,
        connecting signals to buttons should perhaps be replaced by
@@ -875,6 +1072,248 @@ creporter_ui_usecase_dialog (gchar *text)
     return window;
 }
 
+
+static gchar *
+create_server_label_text (gint num_of_checked, gint sum_size, 
+			  gchar * addr)
+{
+    /*
+     * original sum_size is in kilo-bytes. for large values show MB or GB
+     */
+    gfloat shown_size = sum_size;
+    gchar magn = 'k';
+    static gchar *labelstr = NULL;
+
+    if (shown_size > 1024) {
+	magn = 'M';
+	shown_size /= 1024;
+    }
+    if (shown_size > 1024) {
+	magn = 'G';
+	shown_size /= 1024;
+    }
+    if (labelstr) {
+	g_free(labelstr);
+    }
+    labelstr = g_strdup_printf(_("%d selected, %.2f %cB\nServer: %s"), 
+			       num_of_checked, shown_size, magn, addr); 
+    return labelstr;
+}
+
+/**
+  This function is called as callback when one coredump "send enabled" button
+  gets toggled
+  
+  @param b is pointer to button widget
+  @param data is pointer to selection context
+
+  */
+static gboolean 
+select_sending_toggle_one_callback (HildonCheckButton *b, gpointer data) {
+    struct send_sel_data * sel = (struct send_sel_data *)data;
+    gchar new_state;
+
+    if(!sel) {
+	return FALSE;
+    }
+    new_state = (gchar)hildon_check_button_get_active(b);
+    if(new_state && !sel->sel_state) {
+	sel_num_of_checked += 1;
+	sel_sum_size += sel->size;
+    } else if (!new_state && sel->sel_state) {
+	sel_num_of_checked -= 1;
+	sel_sum_size -= sel->size;
+    }
+    sel->sel_state = new_state;
+
+    if (gsettings && glabel_server) {
+	gtk_label_set_label(GTK_LABEL(glabel_server), 
+			    create_server_label_text(sel_num_of_checked, sel_sum_size, 
+						     gsettings->server_addr));
+    }
+
+    if (sel_num_of_checked == 0) {
+	gtk_widget_set_sensitive(selection_send_button, FALSE);
+	gtk_widget_set_sensitive(selection_delete_button, FALSE);
+	gtk_widget_set_sensitive(selection_uncheckall_button, FALSE);
+	gtk_widget_set_sensitive(selection_checkall_button, TRUE);
+    } else {
+	gtk_widget_set_sensitive(selection_uncheckall_button, TRUE);
+	gtk_widget_set_sensitive(selection_send_button, TRUE);
+	gtk_widget_set_sensitive(selection_delete_button, TRUE);
+	if (sel_num_of_checked == num_of_select_send_buttons) {
+	    gtk_widget_set_sensitive(selection_checkall_button, FALSE);
+	} else {
+	    gtk_widget_set_sensitive(selection_checkall_button, TRUE);
+	}
+    }
+    return TRUE;
+}
+
+/* Callback for Select or Deselect ALL button */
+static void
+select_all_button_clicked (GtkWidget *button, void *arg)
+{
+    GSList *temp;
+    struct send_sel_data *sel;
+    gboolean new_state = (gboolean)arg;
+
+    for (temp = files_list; temp ; temp = g_slist_next(temp)) {
+	sel = CORE_SELECT_CTX(temp);
+	if ((sel->sel_state != new_state) && sel->button) {
+	    hildon_check_button_set_active(HILDON_CHECK_BUTTON(sel->button), new_state);
+	}
+    }
+}
+
+/**
+  Create "this system has N core dumps, select for sending" dialog
+  */
+static GtkWidget *
+creporter_ui_select_for_sending_dialog (gint num)
+{
+    static GtkWidget *window;
+    GtkWidget * vbox2;
+    GtkBox * vbox;
+    GtkWidget * pan;
+    GSList *temp = NULL;
+    gchar * s = NULL;
+    struct stat file_info;
+    gchar *title = NULL;
+    struct send_sel_data *sel;
+    gint len;
+ 
+    /* Create a new dialog window for the scrolled window to be
+     * packed into.  */
+    window = gtk_dialog_new ();
+    if (!window) {
+	return NULL;
+    }
+    title = g_strdup_printf(_("Crash Reporter is building list of reports ..."));
+    gtk_window_set_title (GTK_WINDOW (window), title);
+    gtk_window_set_modal (GTK_WINDOW (window), TRUE);
+    //    gtk_widget_set_size_request(window, 700, 380);
+
+    gtk_widget_show_now(window);
+
+    vbox = GTK_BOX((GTK_DIALOG(window))->vbox);
+
+    pan = hildon_pannable_area_new ();
+    g_object_set (G_OBJECT (pan), "initial-hint", TRUE, 
+		  "mov-mode", HILDON_MOVEMENT_MODE_VERT,
+		  "panning_threshold" ,PANNING_THRESHOLD,
+		  NULL);    
+ 
+    gtk_widget_set_size_request (pan, -1, 300); 
+    vbox2 = gtk_vbox_new (FALSE, MARGIN_HALF);      
+
+    for (temp = files_list, num_of_select_send_buttons = 0 ; 
+	 temp && (num_of_select_send_buttons < MAX_SENDING_SHOW); 
+	 temp = g_slist_next(temp)) {
+
+	bzero(&file_info, sizeof(file_info));
+	if(stat(CORE_FILE_NAME(temp), &file_info) == 0) {
+	    gchar *basename;
+	    gchar *basename2;
+	    gchar *psrc;
+	    gchar *pdest;
+	    gchar *p;
+
+	    sel = CORE_SELECT_CTX(temp);
+	    sel->size = file_info.st_size/1024;
+	    sel->button = hildon_check_button_new(HILDON_SIZE_HALFSCREEN_WIDTH | HILDON_SIZE_FINGER_HEIGHT);
+	    hildon_check_button_set_active(HILDON_CHECK_BUTTON(sel->button), FALSE);
+
+	    basename = g_filename_display_basename(CORE_FILE_NAME(temp));
+	    if ((p = g_strrstr(basename, ".rcore.lzo")) != NULL) {
+		*p = 0;
+	    }
+	    if (strlen(basename) > MAX_CORE_NAMELEN_FITS_TO_BUTTON + 2) {
+		p = basename + MAX_CORE_NAMELEN_FITS_TO_BUTTON;
+		*p = *(p+1) = *(p+2) = '.';
+		*(p+3) = 0;
+	    }
+	    /*
+	     * underscores will not show unless doubled, this is GTK button label feature.
+	     * I tried to overwork this with gtk_button_set_use_underline(FALSE)
+	     * but it does not help, buttons will show no undersocres.
+	     * Therefore we will tediously make a longer copy if there is any '_'
+	     */
+	    len = strlen(basename);
+	    if (g_strstr_len(basename, len, "_")) {
+		basename2 = g_malloc(2*len + 1);
+		for (psrc = basename, pdest = basename2 ; *psrc; psrc++, pdest++) {
+		    *pdest = *psrc;
+		    if (*pdest == '_') {
+			*(pdest + 1) = '_';
+			pdest++;
+		    }
+		}
+		*pdest = 0;
+		g_free(basename);
+		basename = basename2;
+	    }
+	    s = g_strdup_printf(_("%s\n%d kB\t%s"), 
+				basename, sel->size, ctime(&file_info.st_mtime));
+	    g_free(basename);
+	    gtk_button_set_label (GTK_BUTTON (sel->button), s);
+	    /*	    gtk_button_set_use_underline(GTK_BUTTON(sel->button), FALSE);*/
+	    hildon_helper_set_logical_font(sel->button, "SmallSystemFont");
+
+	    gtk_box_pack_start (GTK_BOX (vbox2), sel->button, FALSE, FALSE, 0);
+	    g_signal_connect (sel->button, "toggled",
+			      G_CALLBACK (select_sending_toggle_one_callback), sel);
+	    num_of_select_send_buttons += 1;
+	}
+    }
+
+     hildon_pannable_area_add_with_viewport (HILDON_PANNABLE_AREA (pan),
+					     GTK_WIDGET (vbox2));
+     gtk_box_pack_start (vbox, pan, FALSE, FALSE, MARGIN_DEFAULT);
+
+    /*------------*/
+    gsettings = creporter_get_settings();
+    if (gsettings) {
+	glabel_server = gtk_label_new(create_server_label_text(0, 0, 
+							       gsettings->server_addr));
+	//	hildon_helper_set_logical_font(glabel_server, "SmallSystemFont");
+	gtk_label_set_line_wrap(GTK_LABEL(glabel_server), TRUE);
+	gtk_box_pack_start (GTK_BOX (vbox), glabel_server, 
+			    FALSE, FALSE, 0);
+    }
+    /*------------*/
+    /* Add a "Select all" button */
+    selection_checkall_button = gtk_dialog_add_button(GTK_DIALOG(window), _("Select all"),
+						      GTK_RESPONSE_YES);
+    g_signal_connect (G_OBJECT (selection_checkall_button), "clicked",
+		      G_CALLBACK (select_all_button_clicked), (void *)TRUE);
+    
+    /* Add a "Deselect all" button. Note: same handler, different arg for those two */
+    selection_uncheckall_button = gtk_dialog_add_button(GTK_DIALOG(window), _("Deselect all"),
+							GTK_RESPONSE_YES);
+    g_signal_connect (G_OBJECT (selection_uncheckall_button), "clicked",
+		      G_CALLBACK (select_all_button_clicked), FALSE);
+    
+    /* Add a "Send selected" button */
+    selection_send_button = gtk_dialog_add_button(GTK_DIALOG(window), _("Send selected"),
+						  GTK_RESPONSE_OK);
+    
+    /* Add "Delete selected" button */
+    selection_delete_button = gtk_dialog_add_button(GTK_DIALOG(window), 
+						    _("Delete selected"), GTK_RESPONSE_APPLY);
+
+    gtk_widget_set_sensitive(selection_uncheckall_button, FALSE);
+    gtk_widget_set_sensitive(selection_send_button, FALSE);
+    gtk_widget_set_sensitive(selection_delete_button, FALSE);
+    title = g_strdup_printf(_("Crash Reporter has %u stored %s%s"),
+			    num, (num>1)? _("reports") : _("report"),
+			    (num > MAX_SENDING_SHOW)? _(" (not all shown)") : "");
+
+    gtk_window_set_title (GTK_WINDOW (window), title);
+    gtk_widget_show_all (window);
+    return window;
+}
+
 static gboolean
 update_progressbar (gpointer dialog)
 {
@@ -922,9 +1361,9 @@ void creporter_ui_show_progress(char *str, double percent)
 */
 static gboolean creporter_ui_collect_all_corefile_names_at_location(const gchar *location)
 {
-	gchar *filename = NULL;
 	GDir* dir;
 	const gchar* fil;
+	gchar * new_elem = NULL;
 	
 	g_return_val_if_fail(location != NULL, FALSE);
 	
@@ -936,8 +1375,12 @@ static gboolean creporter_ui_collect_all_corefile_names_at_location(const gchar
 		     * location has '/' prepended, so no need to put
 		     * it in between here
 		     */
-		    filename = g_strdup_printf("%s%s",location, fil);
-		    files_list = g_slist_prepend(files_list, filename);
+		    
+		    new_elem = g_malloc0(sizeof(struct send_sel_data) + strlen(location) + strlen(fil) + 1);
+		    strcpy(new_elem + sizeof(struct send_sel_data), location);
+		    strcat(new_elem + sizeof(struct send_sel_data), fil);
+
+		    files_list = g_slist_prepend(files_list, new_elem);
 		}
 	    }
 	    g_dir_close(dir);
